﻿/* 
    Data Community Portugal #1
    Becoming a Carapau of Racing - SQL Server do Zero ao 2017
    2017-07-15 (version 3.01)
    Demo - Working with SQL Server sample commands
    André Melancia, Ricardo Cabral, António Lourenço
    http://Andy.PT

    (C) 2016-2017, André Melancia, Ricardo Cabral, António Lourenço
    
*/

--  +-----------------------------------------------------------------+
--  | MISSION BRIEFING                                                |
--  +-----------------------------------------------------------------+

    -- Run every command separately.
	-- You will need AdventureWorks2014 sample database.
	

--  +-----------------------------------------------------------------+
--  | START HERE!                                                     |
--  +-----------------------------------------------------------------+

-- Hello, this is a comment...

/* Hello, pá...
   this is another comment!
*/

GO -- This is the batch separator. Management Studio will send batches separately.



-- Change context (current database) to database "AdventureWorks2014"
USE AdventureWorks2014;



-- Useful information
SELECT   CURRENT_TIMESTAMP, -- This is the standard, while GETDATE() is SQL Server specific
         GETDATE(),         -- Returns DATETIME data type  
	     SYSDATETIME();     -- Returns DATETIME2 data type (100 ns precision)


-- More useful information
SELECT   CURRENT_USER,      -- Current USER into the current database
         SYSTEM_USER,       -- Current SQL Server instance LOGIN
	     DB_NAME(),         -- Current DATABASE
	     @@SERVERNAME;      -- Current INSTANCE name; 


-- Todos os registos e todos os campos da tabela de pessoas
SELECT   *
FROM     Person.Person;


-- All records, but only some interesting fields
SELECT   BusinessEntityID,
         FirstName,
	     MiddleName,
	     LastName
FROM     Person.Person;


-- Some records only
SELECT   *
FROM     Person.Person
WHERE    FirstName = 'Ken';


-- Some records only (this is NOT a Barbie friendly table)
SELECT   *
FROM     Person.Person
WHERE    FirstName = 'Barbie';


-- Some records only (OR example)
SELECT   *
FROM     Person.Person
WHERE    FirstName = 'Ken'
   or    FirstName = 'Diane';


-- Some records only (AND example, LIKE example)
SELECT   *
FROM     Person.Person
WHERE    FirstName =    'Diane'
   and   LastName  LIKE 'M%'; -- text starts with "M"


-- Ordering stuff
SELECT   *
FROM     Person.Person
WHERE    FirstName = 'Ken'
   or    FirstName = 'Diane'
ORDER BY FirstName, LastName;


-- Ordering stuff descending
SELECT   *
FROM     Person.Person
WHERE    FirstName = 'Ken'
   or    FirstName = 'Diane'
ORDER BY FirstName ASC, LastName DESC;


-- Grouping: Counting first names
SELECT   FirstName,
         Count(*) AS Total
FROM     Person.Person
GROUP BY FirstName
ORDER BY Count(*) DESC;


-- Grouping: Multi grouping example, duplicate detection example
SELECT   FirstName,
         MiddleName,
         LastName,
         Count(*) AS Total
FROM     Person.Person
GROUP BY FirstName, MiddleName, LastName
HAVING   Count(*) > 1;


-- Concatenation
SELECT   FirstName,
         MiddleName,
         LastName,
         FirstName + ' ' + LastName AS FullName
FROM     Person.Person
ORDER BY FirstName, LastName;


-- Concatenation with NULL stuff 
SELECT   FirstName,
         MiddleName,
         LastName,
         FirstName + ' ' + COALESCE(MiddleName, N''   ) + ' ' + LastName AS FullName,
		 FirstName + ' ' + COALESCE(MiddleName, N'☺☺☺') + ' ' + LastName AS FullNameNullsShowing
FROM     Person.Person
ORDER BY FirstName, LastName;



-- Drop table (clean up, only if exists!)
GO
IF (OBJECT_ID('dbo.AvacalhationTable') IS NOT NULL)  DROP TABLE dbo.AvacalhationTable;
GO


-- Creating a table
CREATE TABLE dbo.AvacalhationTable
(
    ID        INT           NOT NULL IDENTITY (1000, 3) PRIMARY KEY,
	Name      NVARCHAR(150) NOT NULL,
	BirthDate DATETIME      NULL,
	Info      NVARCHAR(100) NOT NULL DEFAULT 'Nothing to see here, move along'
);
GO




-- Insert one record
INSERT INTO dbo.AvacalhationTable (Name      )
VALUES                            ('Person 1');


-- View records
SELECT * FROM dbo.AvacalhationTable;


-- Insert another record
INSERT INTO dbo.AvacalhationTable (Name,        BirthDate,    Info                        )
VALUES                            ('Person 2',  '2017-06-13', 'Stuff for person 2 here...');


-- View records
SELECT * FROM dbo.AvacalhationTable;


-- Insert multiple record
INSERT INTO dbo.AvacalhationTable (Name,        BirthDate,    Info                                  )
VALUES                            ('Person 3',  '1903-03-03', 'Stuff 3'                             ),
                                  ('Person 4',  '1904-04-04', ''                                    ),
                                  ('Person 5',  '1905-05-05', DEFAULT                               ),
                                  ('Person 6',  '1906-06-06', 'The current user was ' + CURRENT_USER);


-- View records
SELECT * FROM dbo.AvacalhationTable;


-- Update a record (make sure the ID below exists)
UPDATE dbo.AvacalhationTable
SET    Name = 'UPDATED NAME'
WHERE  ID   = 1012;


-- View records
SELECT * FROM dbo.AvacalhationTable;


-- Delete a record (make sure the ID below exists)
DELETE FROM dbo.AvacalhationTable
WHERE  ID = 1012;


-- View records
SELECT * FROM dbo.AvacalhationTable;


-- Delete ALL records (you can also use DELETE without WHERE, but this is recommended)
TRUNCATE TABLE dbo.AvacalhationTable;


-- Insert the first record after TRUNCATing all records
INSERT INTO dbo.AvacalhationTable (Name      )
VALUES                            ('Person NEW after TRUNCATE');


-- View records
SELECT * FROM dbo.AvacalhationTable; -- Please note the IDENTITY column was reset!


-- Before joining tables, let's see the tables first...
SELECT * FROM Person.EmailAddress;
SELECT * FROM Person.PersonPhone;


-- Joining tables (INNER JOIN: all from Person that also exists in EmailAddress and PersonPhone)
SELECT   p.BusinessEntityID,
         p.FirstName,
         p.LastName,
		 e.EmailAddress,
		 pp.PhoneNumber
FROM     Person.Person       AS p
         INNER JOIN
		 Person.EmailAddress AS e
		 ON p.BusinessEntityID = e.BusinessEntityID
		 INNER JOIN
		 Person.PersonPhone AS pp
		 ON p.BusinessEntityID = pp.BusinessEntityID
WHERE    FirstName = 'Diane'
ORDER BY p.FirstName, p.LastName;


-- Now let's remove an email address (you will need to restore the database to get it back!)
DELETE FROM Person.EmailAddress
WHERE       BusinessEntityID = 1187; -- "Diane Krane"


-- The example above will not show "Diane Krane" (ID 1187). 
-- Try it!
-- Try it!
-- Try it!


-- Joining tables (LEFT JOIN: all from Person and OPTIONALLY the related stuff from EmailAddress and PersonPhone)
SELECT   p.BusinessEntityID,
         p.FirstName,
         p.LastName,
		 e.EmailAddress,
		 pp.PhoneNumber
FROM     Person.Person       AS p
         LEFT JOIN
		 Person.EmailAddress AS e
		 ON p.BusinessEntityID = e.BusinessEntityID
		 LEFT JOIN
		 Person.PersonPhone AS pp
		 ON p.BusinessEntityID = pp.BusinessEntityID
WHERE    FirstName = 'Diane'
ORDER BY p.FirstName, p.LastName;


-- UNION ALL shows all records 
SELECT FirstName, LastName FROM Person.Person WHERE FirstName = 'Ken'
UNION ALL
SELECT FirstName, LastName FROM Person.Person WHERE FirstName = 'Diane';


-- UNION (not ALL) show all records removing duplicates (only one "Ken Sánchez", etc.)
SELECT FirstName, LastName FROM Person.Person WHERE FirstName = 'Ken'
UNION
SELECT FirstName, LastName FROM Person.Person WHERE FirstName = 'Diane';


-- EXCEPT show all records from the first, removing records matching the second
SELECT FirstName, LastName FROM Person.Person WHERE FirstName = 'Ken'
EXCEPT
SELECT FirstName, LastName FROM Person.Person WHERE LastName = 'Kwok';


-- INTESECT (there is a record "Ken Meyer" and a record "Deanna Meyer")
SELECT LastName FROM Person.Person WHERE FirstName = 'Ken'
INTERSECT
SELECT LastName FROM Person.Person WHERE FirstName = 'Deanna';

--  +-----------------------------------------------------------------+
--  | END                                                             |
--  +-----------------------------------------------------------------+
